@@ -144,7 +144,7 @@ c                 icon=1-->no user routine for incident
           write(filen, *) TraceDir(1:klena(TraceDir))//'/trace',
      *         Nevrun + 1
           call kseblk(filen, ' ', leng)
-          call copenfw(IoTrace, filen,  jcon)
+          call copenfw(abs(IoTrace), filen,  jcon)
           if(jcon .ne. 0) then
              call cerrorMsg('**************** Fatal error ', 1)
              call cerrorMsg(
@@ -171,7 +171,7 @@ c      ******************* end of 1 event
 c               user dependent end process
        if(Move.Abort .le. 1) call ue1ev
        if(Trace) then
-          close(IoTrace)
+          close(abs(IoTrace))
        endif
        end
 c      *************  now 1 event is ready to start
@@ -422,6 +422,7 @@ c              until current partile dies or all are put into stack.
 #include  "ZepTrackp.h"
 #include  "ZepTrackv.h"
 #include  "Zcnfig.h"
+#include  "ZepManager.h"
 
       integer icon, info
 
@@ -472,7 +473,10 @@ c                 electric deflection
             endif
 c              take trace info.
             if(Trace) then
-               call epTrace
+               if(IoTrace .lt. 0 .or. (IoTrace .gt. 0 
+     *           .and. cTrack.p.charge .ne. 0)) then
+                  call epTrace
+               endif
             endif
 c              really move
 #ifdef  SUBSTREC
@@ -489,7 +493,10 @@ c                       icon =  1 always.
          else
 c              take trace info. of dead particle.
             if(Trace) then
-               call epTrace
+               if(IoTrace .lt. 0 .or. (IoTrace .gt. 0 
+     *           .and. cTrack.p.charge .ne. 0)) then
+                  call epTrace
+               endif
             endif
 
          endif
@@ -931,6 +938,7 @@ c     ******************
 c     ******************
 c         take trace of paritcles
        implicit none
+#include  "ZepTrackp.h"
 #include  "ZepTrackv.h"
 #include  "Zcode.h"
 #include  "ZepManager.h"
@@ -938,10 +946,10 @@ c         take trace of paritcles
 c          standard Trace information
       record /epPos/ posbw, posw
       real*8 xxx/-1.d37/, yyy/-1.d36/, zzz/1.d34/ 
-      integer kkk/-1000/
+      integer kkk/-1000/, chkchg/-1000/
 
       integer k
-      save xxx, yyy, zzz, kkk
+      save xxx, yyy, zzz, kkk, chkchg
 c
 c          Trace informaion  ccc if for putting only electorns
 c
@@ -952,28 +960,46 @@ c             convert to world coord. starting point of the segment
 ccc      if(k .eq. kelec) then
          if(kkk .ne. k .or. xxx .ne. posbw.x .or. 
      *                      yyy .ne. posbw.y .or.
-     *                      zzz .ne. posbw.z) then
+     *                      zzz .ne. posbw.z .or. 
+     *                   chkchg .ne. cTrack.p.charge ) then
             if(xxx .ne. -1.d37) then
-               write(IoTrace, *)
-               write(IoTrace, *)
+               write(abs(IoTrace), *)
+               write(abs(IoTrace), *)
+            endif
+            if(TimeStruc) then
+               write(abs(IoTrace),
+     *          '(3g16.8, i4, g11.4, i4, g16.8)')
+     *            posbw.x, posbw.y, posbw.z, k,
+     *            Move.Track.p.fm.p(4)-Move.Track.p.mass, 
+     *            cTrack.p.charge,
+     *            cTrack.t
+            else
+               write(abs(IoTrace),'(3g16.8,i4,g11.4, i4)')
+     *            posbw.x, posbw.y, posbw.z, k,
+     *            Move.Track.p.fm.p(4)-Move.Track.p.mass, 
+     *            cTrack.p.charge
             endif
-            write(IoTrace, *) 
-     *        sngl(posbw.x), sngl(posbw.y), sngl(posbw.z), k,
-     *        sngl(cTrack.p.fm.p(4)), cTrack.p.charge
-c     *       , Nevrun+1
-ccc         write(IoTrace, *)  sngl(posbw.x), sngl(posbw.z)
          endif
 c            end point of  the segment
          call epl2w(Cn, Move.Track.pos, posw)
-         write(IoTrace, *) 
-     *       sngl(posw.x), sngl(posw.y), sngl(posw.z), k, 
-     *       sngl(Move.Track.p.fm.p(4)), Move.Track.p.charge
-c     *     ,  Nevrun+1
-ccc      write(IoTrace, *) sngl(posw.x), sngl(posw.z)
+         if(TimeStruc) then
+            write(abs(IoTrace),'(3g16.8,i4,g11.4, i4, 
+     *         g16.8)')
+     *           posw.x, posw.y, posw.z, k,
+     *           Move.Track.p.fm.p(4)-Move.Track.p.mass, 
+     *           Move.Track.p.charge,
+     *           Move.Track.t
+         else
+            write(abs(IoTrace),'(3g16.8,i4,g11.4, i4)')
+     *           posw.x, posw.y, posw.z, k,
+     *           Move.Track.p.fm.p(4)-Move.Track.p.mass, 
+     *           Move.Track.p.charge
+         endif
          xxx = posw.x
          yyy = posw.y
          zzz = posw.z
          kkk = k
+         chkchg = Move.Track.p.charge
 ccc      endif
        end
 c      ****************************************
@@ -1375,22 +1401,34 @@ c      *****************
 c        new position is assumed to be fixed.
 c        (however, before scattering, and energy check).
 c        update time and take trace
+#include  "Zglobalc.h"
 #include  "ZepTrackp.h"
 #include  "ZepTrackv.h"
 
-       real*8 betai    !  1/beta
+       real*8 ctau, u
+       real*8 beta1, betaav
 c
        if(Move.Track.p.mass .eq. 0.) then
-          betai=1.d0
-       elseif(Move.Track.p.fm.p(4)  .gt. Move.Track.p.mass*1000.)then
-          betai=1. + (Move.Track.p.mass /Move.Track.p.fm.p(4))**2/2
-       elseif(Move.Track.p.fm.p(4) .gt. Move.Track.p.mass) then
-          betai=1./
-     *         sqrt(1.d0 - (Move.Track.p.mass/Move.Track.p.fm.p(4))**2)
+          betaav=1.d0
        else
-          betai=1.d8
+          call cgetBeta( Move.Track.p,  betaav)
+          if(betaav .lt. 0.98) then
+             call cgetBeta(cTrack.p, beta1)
+             betaav = (betaav+beta1)/2.
+          endif
+       endif
+       if(betaav .gt. 0.) then
+          Move.Track.t = cTrack.t + Move.dl/betaav
+       else
+c               stopped one. add decay time if possible
+          call cgetctau(cTrack.p, ctau)
+          if(ctau .eq. Infty) then
+             Move.Track.t= cTrack.t+1.e8
+          else
+             call rndc(u)
+             Move.Track.t = cTrack.t - log(u)*ctau
+          endif
        endif
-       Move.Track.t = cTrack.t + Move.dl * betai
        end
 
 c      ************************
